'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _UnauthorizedError = require('../errors/UnauthorizedError');

var _UnauthorizedError2 = _interopRequireDefault(_UnauthorizedError);

var _ForbiddenError = require('../errors/ForbiddenError');

var _ForbiddenError2 = _interopRequireDefault(_ForbiddenError);

var _ValidationError = require('../errors/ValidationError');

var _ValidationError2 = _interopRequireDefault(_ValidationError);

var _nodeFetch = require('node-fetch');

var _nodeFetch2 = _interopRequireDefault(_nodeFetch);

var _NotFoundError = require('../errors/NotFoundError');

var _NotFoundError2 = _interopRequireDefault(_NotFoundError);

var _BlockchainsFacade = require('./facades/BlockchainsFacade');

var _BlockchainsFacade2 = _interopRequireDefault(_BlockchainsFacade);

var _NodesFacade = require('./facades/NodesFacade');

var _NodesFacade2 = _interopRequireDefault(_NodesFacade);

var _UserFacade = require('./facades/UserFacade');

var _UserFacade2 = _interopRequireDefault(_UserFacade);

var _bottleneck = require('bottleneck');

var _bottleneck2 = _interopRequireDefault(_bottleneck);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var apiUrl = process.env.GIN_PLATFORM_API_URL || 'https://api.ginplatform.io';

var handleError = function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(response) {
        var techError, json;
        return regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        techError = 'A technical error occurred';

                        if (!(response.status >= 200 && response.status < 400)) {
                            _context.next = 5;
                            break;
                        }

                        return _context.abrupt('return', response.json());

                    case 5:
                        if (!(response.status === 401)) {
                            _context.next = 9;
                            break;
                        }

                        throw new _UnauthorizedError2.default('Unauthorized!');

                    case 9:
                        if (!(response.status === 403)) {
                            _context.next = 13;
                            break;
                        }

                        throw new _ForbiddenError2.default('Forbidden');

                    case 13:
                        if (!(response.status === 404)) {
                            _context.next = 17;
                            break;
                        }

                        throw new _NotFoundError2.default('Not Found!');

                    case 17:
                        if (!(response.status >= 400 && response.status < 500)) {
                            _context.next = 24;
                            break;
                        }

                        _context.next = 20;
                        return response.json();

                    case 20:
                        json = _context.sent;
                        throw new _ValidationError2.default(json.error || techError);

                    case 24:
                        throw new Error(techError);

                    case 25:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, undefined);
    }));

    return function handleError(_x) {
        return _ref.apply(this, arguments);
    };
}();

var rateLimiter = new _bottleneck2.default({
    minTime: 500,
    maxConcurrent: 2
});

var Client = function () {
    function Client(options) {
        _classCallCheck(this, Client);

        this.apiKey = options.apiKey;
    }

    _createClass(Client, [{
        key: 'request',
        value: function request(endpoint, method, data) {
            var options = {
                method: method || 'GET',
                headers: {
                    'Content-Type': 'application/json',
                    'Authorization': 'Bearer ' + this.apiKey
                }
            };

            if (data) {
                options.body = JSON.stringify(data);
            }

            return rateLimiter.schedule(function () {
                return (0, _nodeFetch2.default)(apiUrl + endpoint, options);
            }).then(handleError);
        }
    }, {
        key: 'get',
        value: function get(endpoint) {
            return this.request(endpoint);
        }
    }, {
        key: 'post',
        value: function post(endpoint, data) {
            return this.request(endpoint, 'POST', data);
        }
    }, {
        key: 'delete',
        value: function _delete(endpoint, data) {
            return this.request(endpoint, 'DELETE', data);
        }
    }, {
        key: 'blockchains',
        get: function get() {
            return new _BlockchainsFacade2.default(this);
        }
    }, {
        key: 'nodes',
        get: function get() {
            return new _NodesFacade2.default(this);
        }
    }, {
        key: 'user',
        get: function get() {
            return new _UserFacade2.default(this);
        }
    }]);

    return Client;
}();

exports.default = Client;