'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _BadParamError = require('../../errors/BadParamError');

var _BadParamError2 = _interopRequireDefault(_BadParamError);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var endpoint = '/nodes';
var uuidValidator = require('uuid-validate');

var validateId = function validateId(id) {
    if (!id || !uuidValidator(id)) {
        throw new _BadParamError2.default('Invalid node ID');
    }
};

var NodesFacade = function () {
    function NodesFacade(client) {
        _classCallCheck(this, NodesFacade);

        this.client = client;
    }

    _createClass(NodesFacade, [{
        key: 'list',
        value: function list() {
            return this.client.get(endpoint);
        }
    }, {
        key: 'create',
        value: function create(data) {
            return this.client.post('' + endpoint, data);
        }
    }, {
        key: 'get',
        value: function get(id) {
            validateId(id);
            return this.client.get(endpoint + '/' + id);
        }
    }, {
        key: 'update',
        value: function update(id, data) {
            validateId(id);
            return this.client.post(endpoint + '/' + id, data);
        }
    }, {
        key: 'delete',
        value: function _delete(id) {
            validateId(id);
            return this.client.delete(endpoint + '/' + id);
        }
    }, {
        key: 'upgrade',
        value: function upgrade(id) {
            validateId(id);
            return this.client.post(endpoint + '/' + id + '/upgrade');
        }
    }, {
        key: 'downgrade',
        value: function downgrade(id) {
            validateId(id);
            return this.client.post(endpoint + '/' + id + '/downgrade');
        }
    }, {
        key: 'rebuild',
        value: function rebuild(id) {
            validateId(id);
            return this.client.post(endpoint + '/' + id + '/rebuild');
        }
    }, {
        key: 'rewards',
        value: function rewards(id, page) {
            validateId(id);
            page = Math.max(page || 1, 1);

            return this.client.get(endpoint + '/' + id + '/rewards?page=' + page);
        }
    }]);

    return NodesFacade;
}();

exports.default = NodesFacade;